from OFS import Folder
from Globals import HTMLFile
from AccessControl import getSecurityManager, Permissions


manage_addImagePagerFolderForm=HTMLFile('imagePagerFolderAddForm', globals())

def manage_addImagePagerFolder(self, id, title,
                           thumbnailsPerPage=10, REQUEST=None):
    """Add a new ImagePagerFolder object with id *id*, title *title*.

    """
    checkPermission=getSecurityManager().checkPermission
    if not checkPermission('Add Folders', self):
        raise 'Unauthorized', (
               'You are not authorized to add ImagePagerFolders.'
               )

    id=str(id)
    title=str(title)

    ob=ImagePagerFolder()
    ob.id=id
    ob.title = title
    self._setObject(id, ob)
    ob=self._getOb(id)

    ob.manage_addProperty('thumbnailsPerPage', int(thumbnailsPerPage), 'int')
    ob.manage_addProperty('pageByFolder', 0, 'int')

    # Add default interface for ImagePagerFolder
    index_html = HTMLFile('index_html', globals())
    text = index_html.read_raw()
    ob.manage_addDTMLMethod(id='index_html', title='')
    idx = ob.index_html
    idx.manage_edit(text, title='')

    # Also add 2nd example interface
    index2_html = HTMLFile('index2_html', globals())
    text2 = index2_html.read_raw()
    ob.manage_addDTMLMethod(id='index2_html', title='')
    idx2 = ob.index2_html
    idx2.manage_edit(text2, title='')

    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)


class ImagePagerFolder(Folder.Folder):
    """Folder which automatically thumbnails and links images.

    """

    meta_type="ImagePagerFolder"
    manage_thumbnailsForm=HTMLFile('thumbnailsForm', globals())

    manage_options=(Folder.Folder.manage_options+
                    ({ "label": "Thumbnails",
                       "action": "manage_thumbnailsForm"},)
                    )

    def manage_setThumbnailsPerPage(self, thumbnailsPerPage=10, REQUEST=None):
        """Set the number of thumbnails show per page."""

        checkPermission=getSecurityManager().checkPermission
        if not checkPermission('Manage properties', self):
            raise 'Unauthorized', ('You are not authorized to change that setting')

        self.thumbnailsPerPage = int(thumbnailsPerPage)

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect('manage_main')


    def manage_setPageByFolder(self, pageByFolder=1, REQUEST=None):
        """Set object to use folders to determine page breaks"""

        checkPermission=getSecurityManager().checkPermission
        if not checkPermission('Manage properties', self):
            raise 'Unauthorized', ('You are not authorized to change that setting')

        if not hasattr(self, 'pageByFolder'):
            ob.manage_addProperty('pageByFolder', 0, 'int')

        self.pageByFolder = int(pageByFolder)

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect('manage_thumbnailsForm')


    def manage_makeThumbnails(self, folderNames=[], size=128,
                               update=0, REQUEST=None):
        """Make thumbnail images of all images in folders in folderNames
        in self. If folderNames is an empty list, do all folders
        which start with the word 'images' or if neither of those
        finds any folders try using self.

        """

        checkPermission=getSecurityManager().checkPermission
        if not checkPermission('Add Folders', self):
            raise 'Unauthorized', ('You are not authorized to make thumbnails')

        folders = self._imageFolders()

        for folder in folders:
            images = folder.objectValues('Image')
            for image in images:
                if hasattr(image, 'original_id'):
                    continue
                original_id = image.getId()
                try:
                    self._makeThumbnail(folder, original_id,
                                           size=size, replace=update)
                except KeyError:
                    # Thumbnail exists, but should not be replaced
                    pass
                except IOError:
                    pass

        if REQUEST is not None:
            REQUEST.RESPONSE.redirect('manage_main')


    def _page(self):
        starts = self._starts()
        if hasattr(self.REQUEST, 'page') and not hasattr(self.REQUEST, 'start'):
            page = int(self.REQUEST['page'])
            if page < 1:
                page = 1
            elif page > len(starts):
                page = len(starts)
        else:
            if hasattr(self.REQUEST, 'start'):
                start = int(self.REQUEST['start'])
                page = 0
                for s in starts:
                    if start > s:
                        page += 1
            else:
                page = 1

        return page


    def getPage(self):
        return self._page()


    def pageThumbnails(self):
        starts = self._starts()
        thumblist = self.thumbnailsList()
        page = self._page()

        if len(starts) > page:
            start = starts[page - 1]
            end = starts[page]
            return thumblist[start:end]
        elif len(starts) == page:
            start = starts[page - 1]
            return thumblist[start:]
        else:
            return []


    def thumbnailsList(self, folder=None):
        if folder is None:
            imageFolders = self._imageFolders()
        else:
            imageFolders = [folder]

        thumblist = []
        for folder in imageFolders:
            images = folder.objectValues('Image')
            for image in images:
                id = image.getId()
                if id.find("_t") >= 0:
                    thumblist.append(image)

        return thumblist


    def _imageFolders(self):
        folders = self.objectValues('Folder')

        imageFolders = []
        for folder in folders:
            id = folder.getId()
            if id.startswith('images'):
                imageFolders.append(folder)
        if not imageFolders:
            imageFolders = [self]

        return imageFolders


    def _starts(self):
        if hasattr(self, 'pageByFolder') and self.pageByFolder:
            imageFolders = self._imageFolders()
            starts = []
            total = 0
            for folder in imageFolders:
                starts.append(total)
                images = self.thumbnailsList(folder)
                total += len(images)
            return starts
        else:
            thumblist = self.thumbnailsList()
            first = 0
            last = len(thumblist)
            skip = self.thumbnailsPerPage
            return range(first, last, skip)


    def nextPage(self):
        page = self._page()
        starts = self._starts()
        if page < len(starts):
            return page + 1
        else:
            return None

    def prevPage(self):
        page = self._page()
        if page > 1:
            return page - 1
        else:
            return None

    def pagesList(self):
        starts = self._starts()
        pages = range(1, len(starts)+1)
        return pages


    def getOriginalImage(self, image):
        """Get original image from thumbnail (using original_id)"""

        f = image.aq_parent
        o = image.original_id

        original = getattr(f, o)

        return original


    def getOriginalURL(self, image):
        """Get original image URL from thumbnail"""

        f = image.aq_parent
        o = image.original_id

        original = getattr(f, o)

        return original.absolute_url()


    def simplePager(self, page=0, REQUEST=None):
        """Return links to thumbnail pages."""
        req = self.REQUEST
        url = req.URL
        seq = self.thumbnailsList()

        starts = self._starts()
        if hasattr(req, 'start'):
            start = req['start']

        pages = self.pagesList()
        currentPage = self.getPage()

        if len(pages) < 2:
            return ""

        printed = ""
        for page in pages:
            if page == currentPage:
                printed += '%s\n' % page
            else:
                printed += '<a href="%s?page=%s&start=%s">%s</a>\n' % (url, page, starts[page-1]+1, page)

        return printed



    def _makeThumbnail(container, self, original_id, newExt='_t', size=128,
                        replace=0):
        """
        Makes a thumbnail image given an image Id when called on a Zope
        folder.

        The thumbnail is a Zope image object that is a small JPG
        representation of the original image. The thumbnail has a
        'original_id' property set to the id of the full size image
        object.
        """

        from PIL import Image
        from cStringIO import StringIO
        import os.path

        # create a thumbnail image file
        original_image = getattr(self, original_id)
        original_file = StringIO(str(original_image.data))
        image = Image.open(original_file)
        image = image.convert('RGB')
        image.thumbnail((int(size),int(size)))
        thumbnail_file = StringIO()
        image.save(thumbnail_file, "JPEG") 
        thumbnail_file.seek(0)

        # create an id for the thumbnail
        path, ext = os.path.splitext(original_id)
        thumbnail_id = path + newExt

        # if there's and old thumbnail, delete it
        if thumbnail_id in self.objectIds():
            if replace:
                self.manage_delObjects(thumbnail_id)
            else:
                raise KeyError, "File exists"

        if original_image.title:
            title = original_image.title
        else:
            title = 'thumbnail image'
 
	
        # create the Zope image object
        self.manage_addProduct['OFSP'].manage_addImage(thumbnail_id,
                                                   thumbnail_file, title)

        thumbnail_image = getattr(self, thumbnail_id)

        # set the 'originial_id' property
        thumbnail_image.manage_addProperty('original_id', 
                                             original_id, 'string')

        #return 'size='+`size`+' path='+path+' newExt='+newExt+' id='+thumbnail_id
